package com.ibm.jumpstart.tvt;

import java.io.*;
import java.lang.reflect.*;
import java.text.*;
import java.util.*;

import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.operation.*;
import org.eclipse.swt.*;
import org.eclipse.swt.dnd.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.part.*;

/**
 * Code or code samples provided herein are provided
 * without warrranty of any kind.  Copyright (c) IBM Corp., 2002
 */

public class PropertiesFileCompareView extends ViewPart {
	private Text sourceDir, targetDir;
	private org.eclipse.swt.widgets.List sourceFileList, targetFileList;
	private Table propertiesTable;
	private Label sourceStatusLine, targetStatusLine;

	static private boolean inResize = false;

	public void setFocus() {
		sourceDir.setFocus();
	}

	protected void selectUntranslatedStrings() {
		TableItem[] items = propertiesTable.getItems();
		propertiesTable.deselectAll();

		for (int i = 0; i < items.length; i++) {
			if (items[i].getText(1).equals(items[i].getText(2)))
				propertiesTable.select(i);
		}
	}

	private void copyToClipboard(String text) {
		Object[] textItems = new Object[1];
		TextTransfer[] textTransfers = new TextTransfer[1];

		textItems[0] = text;
		textTransfers[0] = TextTransfer.getInstance();

		Clipboard clipboard = new org.eclipse.swt.dnd.Clipboard(null);
		clipboard.setContents(textItems, textTransfers);
	}

	private void handleSourceFileListSelection(SelectionEvent e) {
		String[] selectedSourceFiles = sourceFileList.getSelection();

		if (selectedSourceFiles.length == 1) {
			Path path;

			IPath sourcePath = null, targetPath = null;
			String source = null, target = null;
			String sourceExt = null, targetExt = null;

			path = new Path(selectedSourceFiles[0].toLowerCase());
			for (int s = path.segmentCount() - 1; s > 0; s--)
				if (path.segments()[s].equals("eclipse")) {
					sourcePath = path.removeFirstSegments(s);
					sourceExt = sourcePath.getFileExtension();
					sourcePath = sourcePath.removeFileExtension();
					source = sourcePath.lastSegment();
					sourcePath = sourcePath.removeLastSegments(1);
					break;
				}

			if (sourcePath != null && sourceExt != null) {
				String[] targetFiles = targetFileList.getItems();

				for (int i = 0; i < targetFiles.length; i++) {
					path = new Path(targetFiles[i].toLowerCase());
					for (int s = path.segmentCount() - 1; s > 0; s--)
						if (path.segments()[s].equals("eclipse")) {
							targetPath = path.removeFirstSegments(s);
							targetExt = targetPath.getFileExtension();
							targetPath = targetPath.removeFileExtension();
							target = targetPath.lastSegment();
							targetPath = targetPath.removeLastSegments(1);
							break;
						}

					if (targetPath != null
						&& targetExt != null
						&& targetPath.equals(sourcePath)
						&& targetExt.equals(sourceExt)) {

						String ctry = Locale.getDefault().getCountry().toLowerCase();
						if (source.equals(target)
							|| (source + "_" + ctry).equals(target)
							|| (target + "_" + ctry).equals(source)) {
							targetFileList.deselectAll();
							targetFileList.select(i);
							targetFileList.setTopIndex(i);
							break;
						}
					}
				}
			}
		}
	}

	private String formatMessage(String message) {
		Object[] formatObjects =
			new Object[] {
				"<parm1>",
				"<parm2>",
				"<parm3>",
				"<parm4>",
				"<parm5>",
				"<parm6>",
				"<parm7>",
				"<parm8>" };
		if (message.indexOf("{0") >= 0)
			return MessageFormat.format(message, formatObjects);
		else
			return message;
	}

	private void handleFileListDefaultSelection(SelectionEvent e) {
		String[] selectedSourceFiles = sourceFileList.getSelection();
		String[] selectedTargetFiles = targetFileList.getSelection();
		Properties properties1, properties2;
		SortedSet sortedSet = new TreeSet();
		sourceStatusLine.setText(" ");
		targetStatusLine.setText(" ");
		if (selectedSourceFiles.length == 1 && selectedTargetFiles.length == 1) {
			getViewSite().getActionBars().getStatusLineManager().setMessage(
				"File "
					+ (sourceFileList.getSelectionIndex() + 1)
					+ " of "
					+ sourceFileList.getItemCount()
					+ ": "
					+ sourceFileList.getSelection()[0]);
					
			propertiesTable.setVisible(false);
			while (propertiesTable.getItemCount() > 0)
				(propertiesTable.getItem(propertiesTable.getItemCount() - 1))
					.dispose();
			propertiesTable.removeAll();
			propertiesTable.setVisible(true);
			
			properties1 = loadPropertiesFile(selectedSourceFiles[0]);
			properties2 = loadPropertiesFile(selectedTargetFiles[0]);
			if (properties1.size() != properties2.size()) {
				getViewSite()
					.getActionBars()
					.getStatusLineManager()
					.setMessage(
					"Files do not have the same number of keys.  Look for column with \'??? MISSING ???\'.");
			}

			for (Enumeration enum = properties1.propertyNames();
				enum.hasMoreElements(); )
				sortedSet.add((String) enum.nextElement());
			for (Enumeration enum = properties2.propertyNames();
				enum.hasMoreElements(); )
				sortedSet.add((String) enum.nextElement());
				
			for (Iterator i = sortedSet.iterator(); i.hasNext();) {
				String key = (String) i.next();
				TableItem tableItem = new TableItem(propertiesTable, SWT.NULL);
				String value1, value2;
				value1 = properties1.getProperty(key, "??? MISSING ???");
				value2 = properties2.getProperty(key, "??? MISSING ???");
				String[] row =
					new String[] {
						key,
						formatMessage(value1),
						formatMessage(value2)};
				tableItem.setText(row);
			}

			selectUntranslatedStrings();
		}
	}

	private Properties loadPropertiesFile(String filespec) {
		Properties properties = new Properties();
		try {
			properties.load(new FileInputStream(filespec));
		} catch (IOException e) {
		}

		return properties;
	}

	private void fillDirectory(
		String path,
		org.eclipse.swt.widgets.List fileList,
		boolean recursive,
		IProgressMonitor monitor) {

		java.util.List listFiles;
		if (monitor.isCanceled())
			return;
		File directory = new File(path);
		listFiles = Arrays.asList(directory.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return (
					pathname.isDirectory()
						|| pathname.getName().endsWith(".properties"));
			}
		}));
		
		for (Iterator i = listFiles.iterator();
			i.hasNext() && !monitor.isCanceled();
			) {
			final File[] fileArray = new File[1];
			final org.eclipse.swt.widgets.List[] fileListArray =
				new org.eclipse.swt.widgets.List[1];
			fileArray[0] = (File) i.next();
			fileListArray[0] = fileList;
			if (fileArray[0].isDirectory())
				monitor.subTask(fileArray[0].getAbsolutePath());
			if (fileArray[0].isFile()) {
				fileList.getDisplay().syncExec(new Runnable() {
					public void run() {
						fileListArray[0].add(fileArray[0].getAbsolutePath());
					}
				});
			} else if (recursive && fileArray[0].isDirectory())
				fillDirectory(
					fileArray[0].getAbsolutePath(),
					fileListArray[0],
					recursive,
					monitor);
		}
	}

	private void changeDirectory(
		String path,
		org.eclipse.swt.widgets.List fileList) {
		File directory;
		directory = new File(path.trim());
		if (!directory.exists() || !directory.isDirectory()) {
			org.eclipse.jface.dialogs.MessageDialog.openWarning(
				(Shell) null,
				"Properties File Comparison",
				"Path \""
					+ directory.getAbsolutePath()
					+ "\" does not exist or is not a directory.");
			return;
		} else {
			try {
				final File[] directoryArray = new File[1];
				final org.eclipse.swt.widgets.List[] fileListArray =
					new org.eclipse.swt.widgets.List[1];
				directoryArray[0] = directory;
				fileListArray[0] = fileList;
				fileList.removeAll();
				IRunnableWithProgress op = new IRunnableWithProgress() {
					public void run(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {
						monitor.beginTask(
							"Searching...",
							IProgressMonitor.UNKNOWN);
						fillDirectory(
							directoryArray[0].getAbsolutePath(),
							fileListArray[0],
							true,
							monitor);
					}
				};
				new ProgressMonitorDialog(null).run(true, true, op);
			} catch (InvocationTargetException e) { // handle exception
			} catch (InterruptedException e) { // handle cancelation
			}

			SortedSet sortedSet = new TreeSet();
			sortedSet.addAll(Arrays.asList(fileList.getItems()));
			fileList.setItems(
				(String[]) sortedSet.toArray(new String[sortedSet.size()]));
		}
	}

	private void promptForChangeDirectory(
		Text dir,
		org.eclipse.swt.widgets.List fileList) {

		DirectoryDialog dialog =
			new DirectoryDialog(
				getViewSite().getShell(),
				SWT.APPLICATION_MODAL);
		dialog.setFilterPath(dir.getText());
		dialog.setMessage("Enter new directory:");
		dialog.setText("Change Directory");
		String result = dialog.open();
		if (result != null) {
			changeDirectory(result, fileList);
			dir.setText(result);
		}
	}

	public void createPartControl(Composite parent) {
		GridData gridData;
		TableColumn tableColumn;
		TableItem tableItem;
		Label label;
		Color labelForeground =
			Display.getCurrent().getSystemColor(SWT.COLOR_BLUE);
		Composite column1Composite, column2Composite, row1Composite;
		FillLayout fillLayout;
		GridLayout gridLayout;
		Menu menu;
		MenuItem menuItem;
		String stringPref;
		
		gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		parent.setLayout(gridLayout);
		
		row1Composite = new Composite(parent, SWT.NULL);
		fillLayout = new FillLayout();
		fillLayout.type = SWT.HORIZONTAL;
		row1Composite.setLayout(fillLayout);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		row1Composite.setLayoutData(gridData);
		
		column1Composite = new Composite(row1Composite, SWT.NULL);
		gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		column1Composite.setLayout(gridLayout);
		
		column2Composite = new Composite(row1Composite, SWT.NULL);
		gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		column2Composite.setLayout(gridLayout);
		
		label = new Label(column1Composite, SWT.NULL);
		label.setText("Source Language Directory");
		label.setForeground(labelForeground);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		label.setLayoutData(gridData);
		
		sourceDir =
			new Text(
				column1Composite,
				SWT.SINGLE | SWT.BORDER | SWT.HORIZONTAL);
		stringPref =
			TranslationVerificationTestPlugIn
				.getDefault()
				.getDialogSettings()
				.get(
				"sourceDir");
		if (stringPref == null)
			sourceDir.setText("C:\\");
		else
			sourceDir.setText(stringPref);
		sourceDir.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sourceDir.addSelectionListener(new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent e) {
				changeDirectory(sourceDir.getText(), sourceFileList);
			}
		});
		sourceDir.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				sourceDir.selectAll();
			}
		});
		sourceDir.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				TranslationVerificationTestPlugIn
					.getDefault()
					.getDialogSettings()
					.put(
					"sourceDir",
					sourceDir.getText());
			}
		});
		menu = new Menu(sourceDir);
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Change...");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				promptForChangeDirectory(sourceDir, sourceFileList);
			}
		});
		sourceDir.setMenu(menu);
		
		label = new Label(column1Composite, SWT.NULL);
		label.setText("Source Language Files");
		label.setForeground(labelForeground);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		label.setLayoutData(gridData);
		
		sourceFileList =
			new org.eclipse.swt.widgets.List(
				column1Composite,
				SWT.MULTI | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.widthHint = 200;
		sourceFileList.setLayoutData(gridData);
		sourceFileList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleSourceFileListSelection(e);
				handleFileListDefaultSelection(e);
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				handleFileListDefaultSelection(e);
			}
		});
		menu = new Menu(sourceFileList);
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Open");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				try {
					Runtime.getRuntime().exec(
						new String[] {
							"notepad.exe",
							targetFileList.getSelection()[0] });
				} catch (IOException exception) {
				}
			}
		});
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Copy to clipboard");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String[] selection = sourceFileList.getSelection();
				if (selection.length > 0) {
					StringBuffer stringBuffer = new StringBuffer();
					for (int i = 0; i < selection.length; i++) {
						stringBuffer.append(selection[i]);
						if (i < selection.length - 1)
							stringBuffer.append("\n");
					}

					copyToClipboard(stringBuffer.toString());
				}
			}
		});
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Remove");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				sourceFileList.remove(sourceFileList.getSelectionIndices());
			}
		});
		menu.addMenuListener(new MenuAdapter() {
			public void menuShown(MenuEvent e) {
				((Menu) e.widget).getItem(0).setEnabled(
					sourceFileList.getSelectionCount() > 0);
				((Menu) e.widget).getItem(1).setEnabled(
					sourceFileList.getSelectionCount() > 0);
			}
		});
		sourceFileList.setMenu(menu);
		label = new Label(column2Composite, SWT.NULL);
		label.setText("Target Language Directory");
		label.setForeground(labelForeground);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		label.setLayoutData(gridData);
		
		targetDir =
			new Text(
				column2Composite,
				SWT.SINGLE | SWT.BORDER | SWT.HORIZONTAL);
		stringPref =
			TranslationVerificationTestPlugIn
				.getDefault()
				.getDialogSettings()
				.get(
				"targetDir");
		if (stringPref == null)
			targetDir.setText("C:\\");
		else
			targetDir.setText(stringPref);
		targetDir.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		targetDir.addSelectionListener(new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent e) {
				changeDirectory(targetDir.getText(), targetFileList);
			}
		});
		targetDir.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				targetDir.selectAll();
			}
		});
		targetDir.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				TranslationVerificationTestPlugIn
					.getDefault()
					.getDialogSettings()
					.put(
					"targetDir",
					targetDir.getText());
			}
		});
		menu = new Menu(targetDir);
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Change...");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				promptForChangeDirectory(targetDir, targetFileList);
			}
		});
		targetDir.setMenu(menu);
		
		label = new Label(column2Composite, SWT.NULL);
		label.setText("Target Language Files");
		label.setForeground(labelForeground);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		label.setLayoutData(gridData);
		
		targetFileList =
			new org.eclipse.swt.widgets.List(
				column2Composite,
				SWT.MULTI | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.widthHint = 200;
		targetFileList.setLayoutData(gridData);
		targetFileList.addSelectionListener(new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent e) {
				handleFileListDefaultSelection(e);
			}
		});
		menu = new Menu(targetFileList);
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Open");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				try {
					Runtime.getRuntime().exec(
						new String[] {
							"notepad.exe",
							sourceFileList.getSelection()[0] });
				} catch (IOException exception) {
				}
			}
		});
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Copy to clipboard");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String[] selection = targetFileList.getSelection();
				if (selection.length > 0) {
					StringBuffer stringBuffer = new StringBuffer();
					for (int i = 0; i < selection.length; i++) {
						stringBuffer.append(selection[i]);
						if (i < selection.length - 1)
							stringBuffer.append("\n");
					}

					copyToClipboard(stringBuffer.toString());
				}
			}
		});
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Remove");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				targetFileList.remove(targetFileList.getSelectionIndices());
			}
		});
		menu.addMenuListener(new MenuAdapter() {
			public void menuShown(MenuEvent e) {
				((Menu) e.widget).getItem(0).setEnabled(
					targetFileList.getSelectionCount() > 0);
				((Menu) e.widget).getItem(1).setEnabled(
					sourceFileList.getSelectionCount() > 0);
			}
		});
		targetFileList.setMenu(menu);
		
		propertiesTable =
			new Table(
				parent,
				SWT.FULL_SELECTION | SWT.BORDER | SWT.MULTI | SWT.H_SCROLL);
		gridData = new GridData(GridData.FILL_BOTH);
		propertiesTable.setLayoutData(gridData);
		propertiesTable.setHeaderVisible(true);
		propertiesTable.setLinesVisible(true);
		propertiesTable.addControlListener(new ControlAdapter() {
			public void controlResized(ControlEvent e) {
				if (!inResize) {
					inResize = true;
					int width = propertiesTable.getClientArea().width;
					propertiesTable.getColumns()[0].setWidth(
						(width * 20) / 100);
					propertiesTable.getColumns()[1].setWidth(
						(width * 40) / 100);
					propertiesTable.getColumns()[2].setWidth(
						(width * 40) / 100);
					inResize = false;
				}
			}
		});
		propertiesTable.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (propertiesTable.getSelectionCount() == 1) {
					sourceStatusLine.setText(
						propertiesTable.getSelection()[0].getText(1));
					targetStatusLine.setText(
						propertiesTable.getSelection()[0].getText(2));
				} else {
					sourceStatusLine.setText(" ");
					targetStatusLine.setText(" ");
				}
			}
		});
		menu = new Menu(propertiesTable);
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Copy to clipboard");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				TableItem[] selection = propertiesTable.getSelection();
				if (selection.length > 0) {
					StringBuffer stringBuffer = new StringBuffer();
					for (int i = 0; i < selection.length; i++) {
						stringBuffer.append(selection[i].getText(0));
						stringBuffer.append("\t");
						stringBuffer.append(selection[i].getText(1));
						stringBuffer.append("\t");
						stringBuffer.append(selection[i].getText(2));
						if (i < selection.length - 1)
							stringBuffer.append("\n");
					}

					copyToClipboard(stringBuffer.toString());
				}
			}
		});
		menu = new Menu(propertiesTable);
		menuItem = new MenuItem(menu, SWT.PUSH);
		menuItem.setText("Select Untranslated Strings");
		menuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				selectUntranslatedStrings();
			}
		});
		menu.addMenuListener(new MenuAdapter() {
			public void menuShown(MenuEvent e) {
				((Menu) e.widget).getItem(0).setEnabled(
					propertiesTable.getSelectionCount() > 0);
			}
		});
		propertiesTable.setMenu(menu);
		
		tableColumn = new TableColumn(propertiesTable, SWT.NULL);
		tableColumn.setText("Key");
		tableColumn.setResizable(true);
		tableColumn.setWidth(100);
		tableColumn = new TableColumn(propertiesTable, SWT.NULL);
		tableColumn.setResizable(true);
		tableColumn.setWidth(200);
		tableColumn.setText("Source Language");
		tableColumn = new TableColumn(propertiesTable, SWT.NULL);
		tableColumn.setResizable(true);
		tableColumn.setWidth(200);
		tableColumn.setText("Target Language");
		
		sourceStatusLine = new Label(parent, SWT.NULL);
		sourceStatusLine.setText(" ");
		sourceStatusLine.setForeground(labelForeground);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		sourceStatusLine.setLayoutData(gridData);
		
		targetStatusLine = new Label(parent, SWT.NULL);
		targetStatusLine.setText(" ");
		targetStatusLine.setForeground(labelForeground);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		targetStatusLine.setLayoutData(gridData);
	}
}